@@ -6,6 +6,9 @@
  *                         94/01/28  S. Sakai  ver.5.x
  *                         94/03/15  M. Shiotani
  *                         95/03/18  A. Numaguti
+ *
+ *    Copyright (C) 2000-2004 GFD Dennou Club. All rights reserved.
+ *
  */
 
 #include <X11/Xlib.h>
@@ -34,7 +37,7 @@ static int linewidth[MAXWDI] = { 1, 2, 2, 3, 3, 4, 4, 5, 5 };
 static int iwdidz, iclidz, iwtroz, lclatrz, iwnd;
 static int wsxwd, wsywd, wsxmnz, wsxmxz, wsymnz, wsymxz;
 static int ixz, iyz, iwz, ihz, ixxz, iyyz, page;
-static int posx, posy, wait, wait0, wait1, keymask, alternate, dump;
+static int posx, posy, wait, wait0, wait1, keymask, alternate, dump, fgbg;
 static int nbmap, nn1[MAXBMP], nn2[MAXBMP], nx[MAXBMP], ny[MAXBMP];
 static unsigned long pl[1], px[MAXCLI];
 static float rwxold, rwyold;
@@ -49,14 +52,14 @@ static XEvent e;
 /*------------------------- device ------------------------*/
 
 void zxdopn_(width, height, iposx, iposy,
-	     lwait, lwait0, lwait1, lkey, lalt, ldump,
+	     lwait, lwait0, lwait1, lkey, lalt, ldump, lfgbg,
 	     clrmap, cbmmap, file, title)
     int *width, *height, *iposx, *iposy, *lwait, *lwait0, *lwait1,
-	*lkey, *lalt, *ldump;
+	*lkey, *lalt, *ldump, *lfgbg;
     char clrmap[], cbmmap[], file[], title[];
 {
     int bordcl, backcl, ncolor, n, m, msk;
-    long int rx[MAXCLI], gx[MAXCLI], bx[MAXCLI];
+    long int rx[MAXCLI], gx[MAXCLI], bx[MAXCLI], rx1, gx1, bx1;
     char c[80], cmapz[80], bmapz[80];
 
     XSetWindowAttributes att;
@@ -74,6 +77,7 @@ void zxdopn_(width, height, iposx, iposy,
     keymask   = *lkey;
     alternate = *lalt;
     dump      = *ldump;
+    fgbg      = *lfgbg;
 
     cfnchr (dmpfile, file, 79);
     cfnchr (xtitle, title, 79);
@@ -106,6 +110,18 @@ void zxdopn_(width, height, iposx, iposy,
     for (n = 0; n < ncolor; n++)
 	fscanf(stream, "%6ld%6ld%6ld : %s", &rx[n], &gx[n], &bx[n], c);
 
+    if (fgbg) {
+        rx1 = rx[0];
+        gx1 = gx[0];
+        bx1 = bx[0];
+        rx[0] = rx[1];
+        gx[0] = gx[1];
+        bx[0] = bx[1];
+        rx[1] = rx1;
+        gx[1] = gx1;
+        bx[1] = bx1;
+    }
+
 /* read bitmap file */
 
     cfnchr (bmapz, cbmmap, 79);
@@ -402,6 +418,8 @@ void zxgcls_()
 
 /* close graphic segment */
 
+    XFlush(d);
+
 }
 
 /*------------------------- tone --------------------------*/
@@ -411,10 +429,10 @@ void zxgton_(np, wpx, wpy, itpat)
     float wpx[], wpy[];
 {
     static int ltfrst = TRUE, ibitold = -1;
-    static XPoint p[256];
+    static XPoint p[16384];
     int ipx, ipy, iclr, ibit;
     int i, nb;
-    char bitmap[256];
+    char bitmap[16384];
 
     Pixmap pixmap;
     void zxbmcv_(), zxfint_();
@@ -489,14 +507,16 @@ void zxidat_(image, nlen)
     int i;
 
     for (i = 0; i < *nlen; i++) {	
-        XSetForeground (d, gc, px[image[i]]);
-        XDrawPoint (d, ww[iwnd], gc, ixxz, iyyz);
-	ixxz = ixxz + 1;
-	if (ixxz >= ixz+iwz) {
+	if (image[i] != 0) {
+            XSetForeground (d, gc, px[image[i]]);
+            XDrawPoint (d, ww[iwnd], gc, ixxz, iyyz);
+	}
+    	ixxz = ixxz + 1;
+    	if (ixxz >= ixz+iwz) {
 	    ixxz = ixz;
 	    iyyz = iyyz + 1;
 	    if (iyyz % 16 == 0) {
-		printf (".");
+	        printf (".");
 		fflush (stdout);
 	    }
 	}
